'use strict';

Object.defineProperty(exports, "__esModule", {
  value: true
});

var _createClass = function () { function defineProperties(target, props) { for (var i = 0; i < props.length; i++) { var descriptor = props[i]; descriptor.enumerable = descriptor.enumerable || false; descriptor.configurable = true; if ("value" in descriptor) descriptor.writable = true; Object.defineProperty(target, descriptor.key, descriptor); } } return function (Constructor, protoProps, staticProps) { if (protoProps) defineProperties(Constructor.prototype, protoProps); if (staticProps) defineProperties(Constructor, staticProps); return Constructor; }; }();

var _glMatrix = require('gl-matrix');

var _renderer = require('../../renderer');

var _renderer2 = _interopRequireDefault(_renderer);

var _camera = require('../../camera');

var _camera2 = _interopRequireDefault(_camera);

var _mouse = require('../../captors/mouse');

var _mouse2 = _interopRequireDefault(_mouse);

var _quadtree = require('../../quadtree');

var _quadtree2 = _interopRequireDefault(_quadtree);

var _node = require('./programs/node');

var _node2 = _interopRequireDefault(_node);

var _edge = require('./programs/edge');

var _edge2 = _interopRequireDefault(_edge);

var _label = require('../canvas/components/label');

var _label2 = _interopRequireDefault(_label);

var _hover = require('../canvas/components/hover');

var _hover2 = _interopRequireDefault(_hover);

var _utils = require('../../utils');

var _utils2 = require('../utils');

var _utils3 = require('./utils');

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

function _classCallCheck(instance, Constructor) { if (!(instance instanceof Constructor)) { throw new TypeError("Cannot call a class as a function"); } }

function _possibleConstructorReturn(self, call) { if (!self) { throw new ReferenceError("this hasn't been initialised - super() hasn't been called"); } return call && (typeof call === "object" || typeof call === "function") ? call : self; }

function _inherits(subClass, superClass) { if (typeof superClass !== "function" && superClass !== null) { throw new TypeError("Super expression must either be null or a function, not " + typeof superClass); } subClass.prototype = Object.create(superClass && superClass.prototype, { constructor: { value: subClass, enumerable: false, writable: true, configurable: true } }); if (superClass) Object.setPrototypeOf ? Object.setPrototypeOf(subClass, superClass) : subClass.__proto__ = superClass; } /**
                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                * Sigma.js WebGL Renderer
                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                * ========================
                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                *
                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                * File implementing sigma's WebGL Renderer.
                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                */


/**
 * Constants.
 */
var PIXEL_RATIO = (0, _utils2.getPixelRatio)();
var WEBGL_OVERSAMPLING_RATIO = (0, _utils2.getPixelRatio)();

/**
 * Main class.
 *
 * @constructor
 * @param {HTMLElement} container - The graph's container.
 */

var WebGLRenderer = function (_Renderer) {
  _inherits(WebGLRenderer, _Renderer);

  function WebGLRenderer(container) {
    _classCallCheck(this, WebGLRenderer);

    // Validating
    var _this = _possibleConstructorReturn(this, (WebGLRenderer.__proto__ || Object.getPrototypeOf(WebGLRenderer)).call(this));

    if (!(container instanceof HTMLElement)) throw new Error('sigma/renderers/webgl: container should be an html element.');

    // Properties
    _this.sigma = null;
    _this.captors = {};
    _this.container = container;
    _this.elements = {};
    _this.contexts = {};
    _this.listeners = {};

    _this.quadtree = new _quadtree2.default();

    _this.nodeArray = null;
    _this.nodeIndicesArray = null;
    _this.nodeOrder = {};
    _this.nodeDataCache = {};
    _this.edgeArray = null;
    _this.edgeIndicesArray = null;
    _this.edgeOrder = {};

    // TODO: if we drop size scaling => this should become "rescalingFunction"
    _this.nodeRescalingFunction = null;

    // Starting dimensions
    _this.width = 0;
    _this.height = 0;

    // State
    _this.highlightedNodes = new Set();
    _this.hoveredNode = null;
    _this.wasRenderedInThisFrame = false;
    _this.renderFrame = null;
    _this.renderHighlightedNodesFrame = null;
    _this.needToProcess = false;
    _this.needToSoftProcess = false;
    _this.pixel = new Uint8Array(4);

    // Initializing contexts
    _this.createContext('edges');
    _this.createContext('nodes');
    _this.createContext('labels', false);
    _this.createContext('hovers', false);
    _this.createContext('mouse', false);

    // Blending
    var gl = _this.contexts.nodes;

    gl.blendFunc(gl.ONE, gl.ONE_MINUS_SRC_ALPHA);
    gl.enable(gl.BLEND);

    gl = _this.contexts.edges;

    gl.blendFunc(gl.ONE, gl.ONE_MINUS_SRC_ALPHA);
    gl.enable(gl.BLEND);

    // Loading programs
    _this.nodePrograms = {
      def: new _node2.default(_this.contexts.nodes)
    };
    _this.edgePrograms = {
      def: new _edge2.default(_this.contexts.edges)
    };

    // Initial resize
    _this.resize();

    // Initializing the camera
    _this.camera = new _camera2.default({
      width: _this.width,
      height: _this.height
    });

    // Binding camera events
    _this.bindCameraHandlers();

    // Initializing captors
    _this.captors = {
      mouse: new _mouse2.default(_this.elements.mouse, _this.camera)
    };

    // Binding event handlers
    _this.bindEventHandlers();
    return _this;
  }

  /**---------------------------------------------------------------------------
   * Internal methods.
   **---------------------------------------------------------------------------
   */

  /**
   * Method used to test a pixel of the given context.
   *
   * @param  {WebGLContext} gl - Context.
   * @param  {number}       x  - Client x.
   * @param  {number}       y  - Client y.
   * @return {boolean}
   */


  _createClass(WebGLRenderer, [{
    key: 'testPixel',
    value: function testPixel(gl, x, y) {
      (0, _utils3.extractPixel)(gl, x * WEBGL_OVERSAMPLING_RATIO, (this.height - y) * WEBGL_OVERSAMPLING_RATIO, this.pixel);

      return this.pixel[3] !== 0;
    }

    /**
     * Internal function used to create a canvas context and add the relevant
     * DOM elements.
     *
     * @param  {string}  id    - Context's id.
     * @param  {boolean} webgl - Whether the context is a webgl or canvas one.
     * @return {WebGLRenderer}
     */

  }, {
    key: 'createContext',
    value: function createContext(id) {
      var webgl = arguments.length > 1 && arguments[1] !== undefined ? arguments[1] : true;

      var element = (0, _utils2.createElement)('canvas', {
        class: 'sigma-' + id,
        style: {
          position: 'absolute'
        }
      });

      this.elements[id] = element;
      this.container.appendChild(element);

      var contextOptions = {
        preserveDrawingBuffer: true,
        antialias: false
      };

      var context = element.getContext(webgl ? 'webgl' : '2d', contextOptions);

      this.contexts[id] = context;

      return this;
    }

    /**
     * Method binding camera handlers.
     *
     * @return {WebGLRenderer}
     */

  }, {
    key: 'bindCameraHandlers',
    value: function bindCameraHandlers() {
      var _this2 = this;

      this.listeners.camera = function () {
        _this2.scheduleRender();
      };

      this.camera.on('updated', this.listeners.camera);

      return this;
    }

    /**
     * Method binding event handlers.
     *
     * @return {WebGLRenderer}
     */

  }, {
    key: 'bindEventHandlers',
    value: function bindEventHandlers() {
      var _this3 = this;

      // Handling window resize
      this.listeners.handleResize = function () {
        _this3.needToSoftProcess = true;
        _this3.scheduleRender();
      };

      window.addEventListener('resize', this.listeners.handleResize);

      // Function checking if the mouse is on the given node
      var mouseIsOnNode = function mouseIsOnNode(mouseX, mouseY, nodeX, nodeY, size) {
        return mouseX > nodeX - size && mouseX < nodeX + size && mouseY > nodeY - size && mouseY < nodeY + size && Math.sqrt(Math.pow(mouseX - nodeX, 2) + Math.pow(mouseY - nodeY, 2)) < size;
      };

      // Function returning the nodes in the mouse's quad
      var getQuadNodes = function getQuadNodes(mouseX, mouseY) {

        var mouseGraphPosition = _this3.camera.displayToGraph(mouseX, mouseY);

        return _this3.quadtree.point(mouseGraphPosition.x, mouseGraphPosition.y);
      };

      // Handling mouse move
      this.listeners.handleMove = function (e) {

        // NOTE: for the canvas renderer, testing the pixel's alpha should
        // give some boost but this slows things down for WebGL empirically.
        var sizeRatio = Math.pow(_this3.camera.getState().ratio, 0.5);

        var quadNodes = getQuadNodes(e.clientX, e.clientY);

        for (var i = 0, l = quadNodes.length; i < l; i++) {
          var node = quadNodes[i];

          var data = _this3.nodeDataCache[node];

          var pos = _this3.camera.graphToDisplay(data.x, data.y);

          var size = data.size / sizeRatio;

          if (mouseIsOnNode(e.clientX, e.clientY, pos.x, pos.y, size)) {
            _this3.hoveredNode = node;

            _this3.emit('overNode', { node: node });
            return _this3.scheduleHighlightedNodesRender();
          }
        }

        // Checking if the hovered node is still hovered
        if (_this3.hoveredNode) {
          var _data = _this3.nodeDataCache[_this3.hoveredNode];

          var _pos = _this3.camera.graphToDisplay(_data.x, _data.y);

          var _size = _data.size / sizeRatio;

          if (!mouseIsOnNode(e.clientX, e.clientY, _pos.x, _pos.y, _size)) {
            _this3.hoveredNode = null;

            _this3.emit('outNode', { node: _this3.hoveredNode });
            return _this3.scheduleHighlightedNodesRender();
          }
        }
      };

      // Handling down
      this.listeners.handleDown = function (e) {
        var sizeRatio = Math.pow(_this3.camera.getState().ratio, 0.5);

        var quadNodes = getQuadNodes(e.clientX, e.clientY);

        for (var i = 0, l = quadNodes.length; i < l; i++) {
          var node = quadNodes[i];

          var data = _this3.nodeDataCache[node];

          var pos = _this3.camera.graphToDisplay(data.x, data.y);

          var size = data.size / sizeRatio;

          if (mouseIsOnNode(e.clientX, e.clientY, pos.x, pos.y, size)) return _this3.emit('downNode', { node: node });
        }
      };

      // Handling click
      this.listeners.handleClick = function (e) {
        var sizeRatio = Math.pow(_this3.camera.getState().ratio, 0.5);

        var quadNodes = getQuadNodes(e.clientX, e.clientY);

        for (var i = 0, l = quadNodes.length; i < l; i++) {
          var node = quadNodes[i];

          var data = _this3.nodeDataCache[node];

          var pos = _this3.camera.graphToDisplay(data.x, data.y);

          var size = data.size / sizeRatio;

          if (mouseIsOnNode(e.clientX, e.clientY, pos.x, pos.y, size)) return _this3.emit('clickNode', { node: node });
        }

        return _this3.emit('clickStage');
      };

      // TODO: optimize, we don't need to repeat collisions
      this.captors.mouse.on('mousemove', this.listeners.handleMove);
      this.captors.mouse.on('mousedown', this.listeners.handleDown);
      this.captors.mouse.on('click', this.listeners.handleClick);

      return this;
    }

    /**
     * Method binding graph handlers
     *
     * @return {WebGLRenderer}
     */

  }, {
    key: 'bindGraphHandlers',
    value: function bindGraphHandlers() {
      var _this4 = this;

      var graph = this.sigma.getGraph();

      this.listeners.graphUpdate = function () {
        _this4.needToProcess = true;
        _this4.scheduleRender();
      };

      this.listeners.softGraphUpdate = function () {
        _this4.needToSoftProcess = true;
        _this4.scheduleRender();
      };

      // TODO: bind this on composed state events
      // TODO: it could be possible to update only specific node etc. by holding
      // a fixed-size pool of updated items
      graph.on('nodeAdded', this.listeners.graphUpdate);
      graph.on('nodeDropped', this.listeners.graphUpdate);
      graph.on('nodeAttributesUpdated', this.listeners.softGraphUpdate);
      graph.on('edgeAdded', this.listeners.graphUpdate);
      graph.on('nodeDropped', this.listeners.graphUpdate);
      graph.on('edgeAttributesUpdated', this.listeners.softGraphUpdate);
      graph.on('cleared', this.listeners.graphUpdate);

      return this;
    }

    /**
     * Method used to process the whole graph's data.
     *
     * @return {WebGLRenderer}
     */

  }, {
    key: 'process',
    value: function process() {
      var keepArrays = arguments.length > 0 && arguments[0] !== undefined ? arguments[0] : false;


      var graph = this.sigma.getGraph();

      // TODO: possible to index this somehow using two byte arrays or so
      var extent = this.sigma.getGraphExtent();

      // Rescaling function
      this.nodeRescalingFunction = (0, _utils2.createNodeRescalingFunction)({ width: this.width, height: this.height }, extent);

      var minRescaled = this.nodeRescalingFunction({
        x: extent.minX,
        y: extent.minY
      });

      var maxRescaled = this.nodeRescalingFunction({
        x: extent.maxX,
        y: extent.maxY
      });

      this.quadtree.resize({
        x: minRescaled.x,
        y: minRescaled.y,
        width: maxRescaled.x - minRescaled.x,
        height: maxRescaled.y - minRescaled.y
      });

      this.nodeRescaleCache = {};

      var nodeProgram = this.nodePrograms.def;

      if (!keepArrays) {
        this.nodeArray = new Float32Array(_node2.default.POINTS * _node2.default.ATTRIBUTES * graph.order);

        this.nodeOrder = {};
      }

      var nodes = graph.nodes();

      for (var i = 0, l = nodes.length; i < l; i++) {
        var node = nodes[i];

        this.nodeOrder[node] = i;

        var data = this.sigma.getNodeData(node);

        var rescaledData = this.nodeRescalingFunction(data);

        // TODO: Optimize this to be save a loop and one object
        var displayData = (0, _utils.assign)({}, data, rescaledData);

        this.quadtree.add(node, displayData.x, displayData.y, displayData.size);

        this.nodeDataCache[node] = displayData;

        nodeProgram.process(this.nodeArray, displayData, i * _node2.default.POINTS * _node2.default.ATTRIBUTES);
      }

      var edgeProgram = this.edgePrograms.def;

      if (!keepArrays) {
        this.edgeArray = new Float32Array(_edge2.default.POINTS * _edge2.default.ATTRIBUTES * graph.size);

        this.edgeOrder = {};
      }

      var edges = graph.edges();

      for (var _i = 0, _l = edges.length; _i < _l; _i++) {
        var edge = edges[_i];

        this.edgeOrder[edge] = _i;

        var _data2 = this.sigma.getEdgeData(edge),
            extremities = graph.extremities(edge),
            sourceData = this.nodeDataCache[extremities[0]],
            targetData = this.nodeDataCache[extremities[1]];

        edgeProgram.process(this.edgeArray, sourceData, targetData, _data2, _i * _edge2.default.POINTS * _edge2.default.ATTRIBUTES);
      }

      // Computing edge indices if necessary
      if (!keepArrays && typeof edgeProgram.computeIndices === 'function') this.edgeIndicesArray = edgeProgram.computeIndices(this.edgeArray);

      return this;
    }

    /**
     * Method used to process a single node.
     *
     * @return {WebGLRenderer}
     */

  }, {
    key: 'processNode',
    value: function processNode(key) {

      var nodeProgram = this.nodePrograms.def;

      var data = this.sigma.getNodeData(key);

      nodeProgram.process(this.nodeArray, data, this.nodeOrder[key] * _node2.default.POINTS * _node2.default.ATTRIBUTES);

      return this;
    }

    /**
     * Method used to process a single edge.
     *
     * @return {WebGLRenderer}
     */

  }, {
    key: 'processEdge',
    value: function processEdge(key) {

      var graph = this.sigma.getGraph();

      var edgeProgram = this.edgePrograms.def;

      var data = this.sigma.getEdgeData(key),
          extremities = graph.extremities(key),
          sourceData = this.sigma.getNodeData(extremities[0]),
          targetData = this.sigma.getNodeData(extremities[1]);

      edgeProgram.process(this.edgeArray, sourceData, targetData, data, this.edgeOrder[key] * _edge2.default.POINTS * _edge2.default.ATTRIBUTES);

      return this;
    }

    /**---------------------------------------------------------------------------
     * Public API.
     **---------------------------------------------------------------------------
     */

    /**
     * Method used to bind the renderer to a sigma instance.
     *
     * @param  {Sigma} sigma - Target sigma instance.
     * @return {WebGLRenderer}
     */

  }, {
    key: 'bind',
    value: function bind(sigma) {

      // Binding instance
      this.sigma = sigma;

      this.bindGraphHandlers();

      // Processing initial data
      this.process();

      return this;
    }

    /**
     * Method returning the renderer's camera.
     *
     * @return {Camera}
     */

  }, {
    key: 'getCamera',
    value: function getCamera() {
      return this.camera;
    }

    /**
     * Method returning the mouse captor.
     *
     * @return {Camera}
     */

  }, {
    key: 'getMouseCaptor',
    value: function getMouseCaptor() {
      return this.captors.mouse;
    }

    /**
     * Method used to resize the renderer.
     *
     * @param  {number} width  - Target width.
     * @param  {number} height - Target height.
     * @return {WebGLRenderer}
     */

  }, {
    key: 'resize',
    value: function resize(width, height) {
      var previousWidth = this.width,
          previousHeight = this.height;

      if (arguments.length > 1) {
        this.width = width;
        this.height = height;
      } else {
        this.width = this.container.offsetWidth;
        this.height = this.container.offsetHeight;
      }

      // If nothing has changed, we can stop right here
      if (previousWidth === this.width && previousHeight === this.height) return this;

      // Resizing camera
      // TODO: maybe move this elsewhere
      if (this.camera) this.camera.resize({ width: this.width, height: this.height });

      // Sizing dom elements
      for (var id in this.elements) {
        var element = this.elements[id];

        element.style.width = this.width + 'px';
        element.style.height = this.height + 'px';
      }

      // Sizing contexts
      for (var _id in this.contexts) {
        var context = this.contexts[_id];

        // Canvas contexts
        if (context.scale) {
          this.elements[_id].setAttribute('width', this.width * PIXEL_RATIO + 'px');
          this.elements[_id].setAttribute('height', this.height * PIXEL_RATIO + 'px');

          if (PIXEL_RATIO !== 1) context.scale(PIXEL_RATIO, PIXEL_RATIO);
        }

        // WebGL contexts
        else {
            this.elements[_id].setAttribute('width', this.width * WEBGL_OVERSAMPLING_RATIO + 'px');
            this.elements[_id].setAttribute('height', this.height * WEBGL_OVERSAMPLING_RATIO + 'px');
          }

        if (context.viewport) {
          context.viewport(0, 0, this.width * WEBGL_OVERSAMPLING_RATIO, this.height * WEBGL_OVERSAMPLING_RATIO);
        }
      }

      return this;
    }

    /**
     * Method used to clear the canvases.
     *
     * @return {WebGLRenderer}
     */

  }, {
    key: 'clear',
    value: function clear() {
      var context = this.contexts.nodes;
      context.clear(context.COLOR_BUFFER_BIT);

      context = this.contexts.edges;
      context.clear(context.COLOR_BUFFER_BIT);

      context = this.contexts.labels;
      context.clearRect(0, 0, this.width, this.height);

      context = this.contexts.hovers;
      context.clearRect(0, 0, this.width, this.height);

      return this;
    }

    /**
     * Method used to render.
     *
     * @return {WebGLRenderer}
     */

  }, {
    key: 'render',
    value: function render() {

      // If a render was scheduled, we cancel it
      if (this.renderFrame) {
        cancelAnimationFrame(this.renderFrame);
        this.renderFrame = null;
        this.needToProcess = false;
        this.needToSoftProcess = false;
      }

      // First we need to resize
      this.resize();

      // Clearing the canvases
      this.clear();

      // Then we need to extract a matrix from the camera
      var cameraState = this.camera.getState(),
          cameraMatrix = (0, _utils3.matrixFromCamera)(cameraState);

      var translation = _glMatrix.mat3.fromTranslation(_glMatrix.mat3.create(), [this.width / 2, this.height / 2]);

      _glMatrix.mat3.multiply(cameraMatrix, translation, cameraMatrix);

      var program = void 0,
          gl = void 0;

      // Drawing nodes
      gl = this.contexts.nodes;
      program = this.nodePrograms.def;

      // TODO: should probably use another name for the `program` abstraction
      program.render(gl, this.nodeArray, {
        matrix: cameraMatrix,
        width: this.width,
        height: this.height,
        ratio: cameraState.ratio,
        nodesPowRatio: 0.5,
        scalingRatio: WEBGL_OVERSAMPLING_RATIO
      });

      // Drawing edges
      gl = this.contexts.edges;
      program = this.edgePrograms.def;

      program.render(gl, this.edgeArray, {
        matrix: cameraMatrix,
        width: this.width,
        height: this.height,
        ratio: cameraState.ratio,
        edgesPowRatio: 0.5,
        scalingRatio: WEBGL_OVERSAMPLING_RATIO,
        indices: this.edgeIndicesArray
      });

      // Drawing labels
      // TODO: POW RATIO is currently default 0.5 and harcoded
      var nodes = this.sigma.getGraph().nodes(),
          context = this.contexts.labels;

      var sizeRatio = Math.pow(cameraState.ratio, 0.5);

      for (var i = 0, l = nodes.length; i < l; i++) {
        var data = this.nodeDataCache[nodes[i]];

        var _camera$graphToDispla = this.camera.graphToDisplay(data.x, data.y),
            x = _camera$graphToDispla.x,
            y = _camera$graphToDispla.y;

        // TODO: we can cache the labels we need to render until the camera's ratio changes


        var size = data.size / sizeRatio;

        // TODO: this is the label threshold hardcoded
        if (size < 8) continue;

        (0, _label2.default)(context, {
          label: data.label,
          size: size,
          x: x,
          y: y
        });
      }

      // Rendering highlighted nodes
      this.renderHighlightedNodes();

      return this;
    }

    /**
     * Method used to render the highlighted nodes.
     *
     * @return {WebGLRenderer}
     */

  }, {
    key: 'renderHighlightedNodes',
    value: function renderHighlightedNodes() {
      var _this5 = this;

      var camera = this.camera;

      var sizeRatio = Math.pow(camera.getState().ratio, 0.5);

      var context = this.contexts.hovers;

      // Clearing
      context.clearRect(0, 0, this.width, this.height);

      // Rendering
      var render = function render(node) {
        var data = _this5.nodeDataCache[node];

        var _camera$graphToDispla2 = camera.graphToDisplay(data.x, data.y),
            x = _camera$graphToDispla2.x,
            y = _camera$graphToDispla2.y;

        var size = data.size / sizeRatio;

        (0, _hover2.default)(context, {
          label: data.label,
          color: data.color,
          size: size,
          x: x,
          y: y
        });
      };

      if (this.hoveredNode) render(this.hoveredNode);

      this.highlightedNodes.forEach(render);
    }

    /**
     * Method used to schedule a render.
     *
     * @return {WebGLRenderer}
     */

  }, {
    key: 'scheduleRender',
    value: function scheduleRender() {
      var _this6 = this;

      // If we did not render in this frame yet
      // if (!this.wasRenderedInThisFrame) {

      //   // Do we need to process data?
      //   if (this.needToProcess || this.needToSoftProcess)
      //     this.process(this.needToSoftProcess);

      //   // Resetting state
      //   this.renderFrame = null;
      //   this.needToProcess = false;
      //   this.needToSoftProcess = false;

      //   this.render();

      //   this.wasRenderedInThisFrame = true;

      //   requestAnimationFrame(() => {
      //     this.wasRenderedInThisFrame = false;
      //   });

      //   return this;
      // }

      // A frame is already scheduled
      if (this.renderFrame) return this;

      // Let's schedule a frame
      this.renderFrame = requestAnimationFrame(function () {

        // Do we need to process data?
        if (_this6.needToProcess || _this6.needToSoftProcess) _this6.process(_this6.needToSoftProcess);

        // Resetting state
        _this6.renderFrame = null;
        _this6.needToProcess = false;
        _this6.needToSoftProcess = false;

        // Rendering
        _this6.render();
      });
    }

    /**
     * Method used to schedule a hover render.
     *
     * @return {WebGLRenderer}
     */

  }, {
    key: 'scheduleHighlightedNodesRender',
    value: function scheduleHighlightedNodesRender() {
      var _this7 = this;

      if (this.renderHighlightedNodesFrame || this.renderFrame) return this;

      this.renderHighlightedNodesFrame = requestAnimationFrame(function () {

        // Resetting state
        _this7.renderHighlightedNodesFrame = null;

        // Rendering
        _this7.renderHighlightedNodes();
      });
    }

    /**
     * Method used to highlight a node.
     *
     * @param  {string} key - The node's key.
     * @return {WebGLRenderer}
     */

  }, {
    key: 'highlightNode',
    value: function highlightNode(key) {

      // TODO: check the existence of the node
      // TODO: coerce?
      this.highlightedNodes.add(key);

      // Rendering
      this.scheduleHighlightedNodesRender();

      return this;
    }

    /**
     * Method used to unhighlight a node.
     *
     * @param  {string} key - The node's key.
     * @return {WebGLRenderer}
     */

  }, {
    key: 'unhighlightNode',
    value: function unhighlightNode(key) {

      // TODO: check the existence of the node
      // TODO: coerce?
      this.highlightedNodes.delete(key);

      // Rendering
      this.scheduleHighlightedNodesRender();

      return this;
    }
  }]);

  return WebGLRenderer;
}(_renderer2.default);

exports.default = WebGLRenderer;