'use strict';

Object.defineProperty(exports, "__esModule", {
  value: true
});

var _createClass = function () { function defineProperties(target, props) { for (var i = 0; i < props.length; i++) { var descriptor = props[i]; descriptor.enumerable = descriptor.enumerable || false; descriptor.configurable = true; if ("value" in descriptor) descriptor.writable = true; Object.defineProperty(target, descriptor.key, descriptor); } } return function (Constructor, protoProps, staticProps) { if (protoProps) defineProperties(Constructor.prototype, protoProps); if (staticProps) defineProperties(Constructor, staticProps); return Constructor; }; }();

var _program = require('./program');

var _program2 = _interopRequireDefault(_program);

var _utils = require('../utils');

var _edgeFastVert = require('../shaders/edge.fast.vert.glsl');

var _edgeFastVert2 = _interopRequireDefault(_edgeFastVert);

var _edgeFastFrag = require('../shaders/edge.fast.frag.glsl');

var _edgeFastFrag2 = _interopRequireDefault(_edgeFastFrag);

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

function _classCallCheck(instance, Constructor) { if (!(instance instanceof Constructor)) { throw new TypeError("Cannot call a class as a function"); } }

function _possibleConstructorReturn(self, call) { if (!self) { throw new ReferenceError("this hasn't been initialised - super() hasn't been called"); } return call && (typeof call === "object" || typeof call === "function") ? call : self; }

function _inherits(subClass, superClass) { if (typeof superClass !== "function" && superClass !== null) { throw new TypeError("Super expression must either be null or a function, not " + typeof superClass); } subClass.prototype = Object.create(superClass && superClass.prototype, { constructor: { value: subClass, enumerable: false, writable: true, configurable: true } }); if (superClass) Object.setPrototypeOf ? Object.setPrototypeOf(subClass, superClass) : subClass.__proto__ = superClass; } /**
                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                * Sigma.js WebGL Renderer Node Program
                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                * =====================================
                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                *
                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                * Program rendering edges using GL_LINES which is presumably very fast but
                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                * won't render thickness correctly on some GPUs and has some quirks.
                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                */


var EdgeFastProgram = function (_Program) {
  _inherits(EdgeFastProgram, _Program);

  function EdgeFastProgram() {
    _classCallCheck(this, EdgeFastProgram);

    var _this = _possibleConstructorReturn(this, (EdgeFastProgram.__proto__ || Object.getPrototypeOf(EdgeFastProgram)).call(this));

    _this.vertexShaderSource = _edgeFastVert2.default;
    _this.fragmentShaderSource = _edgeFastFrag2.default;
    return _this;
  }

  _createClass(EdgeFastProgram, [{
    key: 'process',
    value: function process(array, sourceData, targetData, data, i) {

      if (sourceData.hidden || targetData.hidden || data.hidden) {
        for (var l = i + EdgeFastProgram.POINTS * EdgeFastProgram.ATTRIBUTES; i < l; i++) {
          array[i] = 0;
        }
      }

      var thickness = data.size || 1,
          x1 = sourceData.x,
          y1 = sourceData.y,
          x2 = targetData.x,
          y2 = targetData.y,
          color = (0, _utils.floatColor)(data.color);

      array[i++] = x1;
      array[i++] = y1;
      array[i++] = color;

      array[i++] = x2;
      array[i++] = y2;
      array[i++] = color;
    }
  }, {
    key: 'render',
    value: function render(gl, array, params) {
      var program = this.program;
      gl.useProgram(program);

      // Attribute locations
      var colorLocation = gl.getAttribLocation(program, 'a_color'),
          positionLocation = gl.getAttribLocation(program, 'a_position'),
          resolutionLocation = gl.getUniformLocation(program, 'u_resolution'),
          matrixLocation = gl.getUniformLocation(program, 'u_matrix');

      var buffer = gl.createBuffer();
      gl.bindBuffer(gl.ARRAY_BUFFER, buffer);
      gl.bufferData(gl.ARRAY_BUFFER, array, gl.DYNAMIC_DRAW);

      gl.uniform2f(resolutionLocation, params.width, params.height);
      gl.uniformMatrix3fv(matrixLocation, false, params.matrix);

      gl.enableVertexAttribArray(positionLocation);
      gl.enableVertexAttribArray(colorLocation);

      gl.vertexAttribPointer(positionLocation, 2, gl.FLOAT, false, EdgeFastProgram.ATTRIBUTES * Float32Array.BYTES_PER_ELEMENT, 0);
      gl.vertexAttribPointer(colorLocation, 1, gl.FLOAT, false, EdgeFastProgram.ATTRIBUTES * Float32Array.BYTES_PER_ELEMENT, 8);

      // TODO: use gl line thickness
      gl.lineWidth(3);
      gl.drawArrays(gl.LINES, 0, array.length / EdgeFastProgram.ATTRIBUTES);
    }
  }]);

  return EdgeFastProgram;
}(_program2.default);

exports.default = EdgeFastProgram;


EdgeFastProgram.POINTS = 2;
EdgeFastProgram.ATTRIBUTES = 3;