'use strict';

Object.defineProperty(exports, "__esModule", {
  value: true
});

var _createClass = function () { function defineProperties(target, props) { for (var i = 0; i < props.length; i++) { var descriptor = props[i]; descriptor.enumerable = descriptor.enumerable || false; descriptor.configurable = true; if ("value" in descriptor) descriptor.writable = true; Object.defineProperty(target, descriptor.key, descriptor); } } return function (Constructor, protoProps, staticProps) { if (protoProps) defineProperties(Constructor.prototype, protoProps); if (staticProps) defineProperties(Constructor, staticProps); return Constructor; }; }();

var _program = require('./program');

var _program2 = _interopRequireDefault(_program);

var _utils = require('../utils');

var _nodeVert = require('../shaders/node.vert.glsl');

var _nodeVert2 = _interopRequireDefault(_nodeVert);

var _nodeFrag = require('../shaders/node.frag.glsl');

var _nodeFrag2 = _interopRequireDefault(_nodeFrag);

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

function _classCallCheck(instance, Constructor) { if (!(instance instanceof Constructor)) { throw new TypeError("Cannot call a class as a function"); } }

function _possibleConstructorReturn(self, call) { if (!self) { throw new ReferenceError("this hasn't been initialised - super() hasn't been called"); } return call && (typeof call === "object" || typeof call === "function") ? call : self; }

function _inherits(subClass, superClass) { if (typeof superClass !== "function" && superClass !== null) { throw new TypeError("Super expression must either be null or a function, not " + typeof superClass); } subClass.prototype = Object.create(superClass && superClass.prototype, { constructor: { value: subClass, enumerable: false, writable: true, configurable: true } }); if (superClass) Object.setPrototypeOf ? Object.setPrototypeOf(subClass, superClass) : subClass.__proto__ = superClass; } /**
                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                * Sigma.js WebGL Renderer Node Program
                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                * =====================================
                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                *
                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                * Simple program rendering nodes as discs, shaped by triangles using the
                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                * `gl.TRIANGLES` display mode. So, to draw one node, it will need to store
                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                * three times the center of the node, with the color, the size and an angle
                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                * indicating which "corner" of the triangle to draw.
                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                */


var ANGLE_1 = 0,
    ANGLE_2 = 2 * Math.PI / 3,
    ANGLE_3 = 4 * Math.PI / 3;

var NodeProgram = function (_Program) {
  _inherits(NodeProgram, _Program);

  function NodeProgram(gl) {
    _classCallCheck(this, NodeProgram);

    // Initializing buffers
    var _this = _possibleConstructorReturn(this, (NodeProgram.__proto__ || Object.getPrototypeOf(NodeProgram)).call(this, gl, _nodeVert2.default, _nodeFrag2.default));

    _this.buffer = gl.createBuffer();
    gl.bindBuffer(gl.ARRAY_BUFFER, _this.buffer);

    // Locations
    _this.positionLocation = gl.getAttribLocation(_this.program, 'a_position');
    _this.sizeLocation = gl.getAttribLocation(_this.program, 'a_size');
    _this.colorLocation = gl.getAttribLocation(_this.program, 'a_color');
    _this.angleLocation = gl.getAttribLocation(_this.program, 'a_angle');
    _this.resolutionLocation = gl.getUniformLocation(_this.program, 'u_resolution');
    _this.matrixLocation = gl.getUniformLocation(_this.program, 'u_matrix');
    _this.ratioLocation = gl.getUniformLocation(_this.program, 'u_ratio');
    _this.scaleLocation = gl.getUniformLocation(_this.program, 'u_scale');

    // Bindings
    gl.enableVertexAttribArray(_this.positionLocation);
    gl.enableVertexAttribArray(_this.sizeLocation);
    gl.enableVertexAttribArray(_this.colorLocation);
    gl.enableVertexAttribArray(_this.angleLocation);

    gl.vertexAttribPointer(_this.positionLocation, 2, gl.FLOAT, false, NodeProgram.ATTRIBUTES * Float32Array.BYTES_PER_ELEMENT, 0);

    gl.vertexAttribPointer(_this.sizeLocation, 1, gl.FLOAT, false, NodeProgram.ATTRIBUTES * Float32Array.BYTES_PER_ELEMENT, 8);

    gl.vertexAttribPointer(_this.colorLocation, 1, gl.FLOAT, false, NodeProgram.ATTRIBUTES * Float32Array.BYTES_PER_ELEMENT, 12);

    gl.vertexAttribPointer(_this.angleLocation, 1, gl.FLOAT, false, NodeProgram.ATTRIBUTES * Float32Array.BYTES_PER_ELEMENT, 16);
    return _this;
  }

  _createClass(NodeProgram, [{
    key: 'process',
    value: function process(array, data, i) {
      var color = (0, _utils.floatColor)(data.color);

      array[i++] = data.x;
      array[i++] = data.y;
      array[i++] = data.size;
      array[i++] = color;
      array[i++] = ANGLE_1;

      array[i++] = data.x;
      array[i++] = data.y;
      array[i++] = data.size;
      array[i++] = color;
      array[i++] = ANGLE_2;

      array[i++] = data.x;
      array[i++] = data.y;
      array[i++] = data.size;
      array[i++] = color;
      array[i++] = ANGLE_3;
    }
  }, {
    key: 'render',
    value: function render(gl, array, params) {
      var program = this.program;
      gl.useProgram(program);

      gl.bufferData(gl.ARRAY_BUFFER, array, gl.DYNAMIC_DRAW);

      gl.uniform2f(this.resolutionLocation, params.width, params.height);
      gl.uniform1f(this.ratioLocation, 1 / Math.pow(params.ratio, params.nodesPowRatio));
      gl.uniform1f(this.scaleLocation, params.scalingRatio);
      gl.uniformMatrix3fv(this.matrixLocation, false, params.matrix);

      gl.drawArrays(gl.TRIANGLES, 0, array.length / NodeProgram.ATTRIBUTES);
    }
  }]);

  return NodeProgram;
}(_program2.default);

exports.default = NodeProgram;


NodeProgram.POINTS = 3;
NodeProgram.ATTRIBUTES = 5;