'use strict';

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.floatColor = floatColor;
exports.matrixFromCamera = matrixFromCamera;
exports.extractPixel = extractPixel;

var _glMatrix = require('gl-matrix');

/**
 * Memoized function returning a float-encoded color from various string
 * formats describing colors.
 */
var FLOAT_COLOR_CACHE = {}; /**
                             * Sigma.js WebGL Renderer Utils
                             * ==============================
                             *
                             * Miscelleanous helper functions used by sigma's WebGL renderer.
                             */

// TODO: this is heavy for what we do with it


var RGBA_TEST_REGEX = /^\s*rgba?\s*\(/;
var RGBA_EXTRACT_REGEX = /^\s*rgba?\s*\(\s*([0-9]*)\s*,\s*([0-9]*)\s*,\s*([0-9]*)\s*(,.*)?\)\s*$/;

function floatColor(val) {

  // If the color is already computed, we yield it
  if (typeof FLOAT_COLOR_CACHE[val] !== 'undefined') return FLOAT_COLOR_CACHE[val];

  var r = 0,
      g = 0,
      b = 0;

  // Handling hexadecimal notation
  if (val[0] === '#') {
    if (val.length === 4) {
      r = parseInt(val.charAt(1) + val.charAt(1), 16);
      g = parseInt(val.charAt(2) + val.charAt(2), 16);
      b = parseInt(val.charAt(3) + val.charAt(3), 16);
    } else {
      r = parseInt(val.charAt(1) + val.charAt(2), 16);
      g = parseInt(val.charAt(3) + val.charAt(4), 16);
      b = parseInt(val.charAt(5) + val.charAt(6), 16);
    }
  }

  // Handling rgb notation
  else if (RGBA_TEST_REGEX.test(val)) {
      var match = val.match(RGBA_EXTRACT_REGEX);

      r = +match[1];
      g = +match[2];
      b = +match[3];
    }

  var color = r * 256 * 256 + g * 256 + b;

  FLOAT_COLOR_CACHE[val] = color;

  return color;
}

/**
 * Function returning a matrix from the current state of the camera.
 */

// TODO: it's possible to optimize this drastically!
function matrixFromCamera(state) {
  var angle = state.angle,
      ratio = state.ratio,
      x = state.x,
      y = state.y;


  var matrix = _glMatrix.mat3.create(),
      scale = _glMatrix.mat3.fromScaling(_glMatrix.mat3.create(), [1 / ratio, 1 / ratio]),
      rotation = _glMatrix.mat3.fromRotation(_glMatrix.mat3.create(), -angle),
      translation = _glMatrix.mat3.fromTranslation(_glMatrix.mat3.create(), [-x, -y]);

  _glMatrix.mat3.multiply(matrix, scale, rotation);
  _glMatrix.mat3.multiply(matrix, matrix, translation);

  return matrix;
}

/**
 * Function extracting the color at the given pixel.
 */
function extractPixel(gl, x, y, array) {
  var data = array || new Uint8Array(4);

  gl.readPixels(x, y, 1, 1, gl.RGBA, gl.UNSIGNED_BYTE, data);

  return data;
}